#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
# Settings
#"""

import datavar
import sys
import koprdial
from PyQt4 import QtCore, QtGui

font = datavar.DefaultFont()

# Main dialog with settings tabs
class TabDialog(QtGui.QDialog):
    def __init__(self, settings, cat, observers, returns, parent=None):
        super(TabDialog, self).__init__(parent)
        self.settings = settings
        self.cat = cat
        self.obscode = observers
        self.returns = returns

        tabWidget = QtGui.QTabWidget()

        self.observers = ListTab(self.obscode, 'obs')
        self.peri = ListTab(self.returns, 'peri')
        self.types = ItemTab(self.cat, 'type')

        self.SetTab = MainSetTab(self.settings[4:], [ 'Perihelion asymetry',
                                                      'Apply personal coefficients',
                                                      'Phase correction',
                                                      'Forward scattering correction, Dust-to-Gas ratio:', 
                                                      '', 
                                                      'Fixed slope, with n = :',
                                                      '',
                                                      'Bortle survival limit (only for LP comets)'] )
        self.ExclTab = GeneralTab(self.settings[:4], [ 'Uncertain observations',
                                                       'Negative observations',
                                                       'Border observations +/-'])
        tabWidget.addTab(self.SetTab, 'Settings')
        tabWidget.addTab(self.ExclTab, 'Exclude')
        tabWidget.addTab(self.types, 'Types')
        tabWidget.addTab(self.peri, 'Returns')
        tabWidget.addTab(self.observers, 'Observers')
        tabWidget.setFont(font)

        buttonBox = QtGui.QDialogButtonBox(QtGui.QDialogButtonBox.Ok | QtGui.QDialogButtonBox.Cancel)

        buttonBox.accepted.connect(self.save_setup)
        buttonBox.rejected.connect(self.reject)
        
        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(tabWidget)
        vbox.addWidget(buttonBox)
        self.setLayout(vbox)
        self.setGeometry(200, 200, 400, 400)
        self.setWindowTitle('Analysis Settings and Selection')

    def save_setup(self):
        self.settings = []
        for i in range(len(self.ExclTab.elements)-1):
          self.settings.append(int(self.ExclTab.elements[i].checkState())/2)
        self.settings.append(float(self.ExclTab.elements[i+1].text()))

        for i in (range(8)):
          if i in (4,6): 
            self.settings.append(float(self.SetTab.elements[i].text()))
          else:
            self.settings.append(int(self.SetTab.elements[i].checkState())/2)
        
        self.cat = []
        for key in self.types.activ.keys(): 
           self.cat.append( [ str(self.types.activ[key].text()), int(self.types.activ[key].checkState())/2, str(self.types.color[key].currentText()) ] )
         
        self.returns = self.peri.items
          
        self.obscode = []
        for index in xrange(self.observers.listEnabled.count()):
          self.obscode.append( [ str(self.observers.listEnabled.item(index).text()),1] )
        for index in xrange(self.observers.listDisabled.count()):
          self.obscode.append( [ str(self.observers.listDisabled.item(index).text()),0] )
        
        self.close()

#Tab with general settings
class MainSetTab(QtGui.QWidget):
    def __init__(self, settings, labels, parent=None):
        super(MainSetTab, self).__init__(parent)
        self.settings = settings
        self.elements = []
        
        grid = QtGui.QGridLayout()
        for i, item in enumerate(labels):
          if i in (4,6):
            self.elements.append( QtGui.QLineEdit(self) )
            self.elements[i].setFont(font)
            self.elements[i].setText(str(self.settings[i]))
            grid.addWidget(self.elements[i], i, 0)

          else:
            self.elements.append( QtGui.QCheckBox(item,self) )
            self.elements[i].setFont(font)
            self.elements[i].setCheckState(int(self.settings[i]*2))
            grid.addWidget(self.elements[i], i, 0)
        
        self.elements[i].stateChanged.connect(self.Enable)

#        if not self.elements[i].isChecked():
#          self.elements[i+1].setDisabled(1)   

        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(grid)
        hbox.addStretch(1)
        
        vbox = QtGui.QVBoxLayout()
        vbox.addLayout(hbox)
        vbox.addStretch(1) 
        
        self.setLayout(vbox)
 
    def Enable(self, state):
        if state == QtCore.Qt.Checked:
          self.elements[-2].setDisabled(0)
        else:
          self.elements[-2].setDisabled(1)

class GeneralTab(QtGui.QWidget):
    def __init__(self, settings, labels, parent=None):
        super(GeneralTab, self).__init__(parent)
        self.settings = settings
        self.elements = []
        
        grid = QtGui.QGridLayout()
        for i, item in enumerate(labels):
          self.elements.append( QtGui.QCheckBox(item,self) )

          self.elements[i].setFont(font)
          self.elements[i].setCheckState(int(self.settings[i]*2))
          grid.addWidget(self.elements[i], i, 0)
        
        self.elements[i].stateChanged.connect(self.Enable)
        
        self.elements.append( QtGui.QLineEdit(self) )
        self.elements[i+1].setFont(font)
        self.elements[i+1].setText(str(self.settings[i+1]))
        grid.addWidget(self.elements[i+1], i+1, 0)

        if not self.elements[i].isChecked():
          self.elements[i+1].setDisabled(1)   
        
        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(grid)
        hbox.addStretch(1)
        
        vbox = QtGui.QVBoxLayout()
        vbox.addLayout(hbox)
        vbox.addStretch(1) 
        
        self.setLayout(vbox)
 
    def Enable(self, state):
        if state == QtCore.Qt.Checked:
          self.elements[-1].setDisabled(0)
        else:
          self.elements[-1].setDisabled(1)

class ListTab(QtGui.QWidget):
    def __init__(self, items, typ, parent=None):
        super(ListTab, self).__init__(parent)
        self.items = sorted(items, key=lambda x: x[0])
        self.typ = typ
                
        hbox = QtGui.QHBoxLayout()
        self.listEnabled = QtGui.QListWidget()
        self.listEnabled.setFont(font)
        self.listEnabled.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)
        
        self.listDisabled = QtGui.QListWidget()
        self.listDisabled.setFont(font)
        self.listDisabled.setSelectionMode(QtGui.QAbstractItemView.ExtendedSelection)

        for item in self.items:
          if item[1] == 1:
             self.listEnabled.addItem(item[0])
          else:
             self.listDisabled.addItem(item[0])
        
        vbox = QtGui.QVBoxLayout()
        label = QtGui.QLabel('Included')
        label.setFont(font)
        vbox.addWidget(label)
        vbox.addWidget(self.listEnabled)
        hbox.addLayout(vbox)

        self.btnexcl = QtGui.QPushButton('Exclude >>', self)
        self.btnexcl.setFont(font)
        self.btnexcl.clicked.connect(self.button_click_disable)
        self.btnincl = QtGui.QPushButton('<< Include', self)
        self.btnincl.setFont(font)
        self.btnincl.clicked.connect(self.button_click_enable)

        vbox = QtGui.QVBoxLayout()
        label = QtGui.QLabel('')
        label.setFont(font)
        vbox.addWidget(label)
        vbox.addWidget(self.btnexcl)
        vbox.addWidget(self.btnincl)
        if typ == 'peri':
          self.btncol = QtGui.QPushButton('Change color', self)
          self.btncol.setFont(font)
          self.btncol.clicked.connect(self.button_click_color)
          vbox.addWidget(self.btncol)
        vbox.addStretch(1)
        hbox.addLayout(vbox)
        
        vbox = QtGui.QVBoxLayout()
        label = QtGui.QLabel('Excluded')
        label.setFont(font)
        vbox.addWidget(label)
        vbox.addWidget(self.listDisabled)
        hbox.addLayout(vbox)

        self.setLayout(hbox)
    
    def button_click_color(self):
        rows = sorted([index.row() for index in self.listEnabled.selectedIndexes()], reverse=True)
        if len(rows) > 1:
          koprdial.InfoDialog(1, 'Too many items', 'Select only one return to edit color!')
        elif len(rows) == 0:
          koprdial.InfoDialog(1, 'No item selected', 'Select one return to edit color!')
        else:
          year = str(self.listEnabled.currentItem().text())
          colors = [x[2] for x in self.items if x[1] == 1]
          for index, line in enumerate(self.items):
            if line[0] == year:
              actual = line[2]
              break
          
          colors = [x for x in datavar.Colorlist() if x not in colors]
          colors.append(actual)
          a = Selector(year, actual, colors)
          a.exec_()
          self.items[index][2] = a.color

    def button_click_disable(self):
        rows = sorted([index.row() for index in self.listEnabled.selectedIndexes()], reverse=True)
        items = []       
        for row in rows:
          if self.typ == 'peri':
            items.append(self.listEnabled.item(row).text())
          self.listDisabled.addItem(self.listEnabled.takeItem(row))

        if self.typ == 'peri':
          for item in self.items:
            if item[0] in items:
              item[1], item[2] = 0, ''
            
        self.listDisabled.sortItems()
        
    def button_click_enable(self):
        rows = sorted([index.row() for index in self.listDisabled.selectedIndexes()], reverse=True)       
        items = []       
        for row in rows:
          if self.typ == 'peri':
            items.append(self.listDisabled.item(row).text())
          self.listEnabled.addItem(self.listDisabled.takeItem(row))

        if self.typ == 'peri':
          colors = [x[2] for x in self.items if x[1] == 1]
          colors = [x for x in datavar.Colorlist() if x not in colors]
          for item in self.items:
            if item[0] in items:
              item[1], item[2] = 1, colors[-1]
              colors.pop()
            
        self.listEnabled.sortItems()
        
class ItemTab(QtGui.QWidget):
    def __init__(self, items, typ, parent=None):
        super(ItemTab, self).__init__(parent)
        self.output, self.typ, colors = sorted(items, key=lambda x: x[0]), typ, [] 
        grid = QtGui.QGridLayout()
        
        if self.typ == 'peri':
          colors = datavar.Colorlist()
        elif self.typ == 'type': 
          for i in range (len(datavar.Markers())):
            colors.append(datavar.Markers()[i][0]+','+datavar.Markers()[i][1])

        self.activ, self.color, curindex, i = {}, {}, 0, 0  

        for i in range(len(self.output)):
            self.activ[i] = QtGui.QCheckBox(self.output[i][0], self)
            self.activ[i].setFont(font)
            self.activ[i].setCheckState(self.output[i][1]*2) 
            grid.addWidget(self.activ[i], *(i,0))

            self.color[i] = QtGui.QComboBox(self)
            self.color[i].setStyleSheet("selection-color: black")
            for j in range(len(colors)):
              self.color[i].addItem(colors[j])
              if colors[j] == self.output[i][2]:
                curindex = j
            self.color[i].setFont(font)
            self.color[i].setCurrentIndex(curindex)            
            grid.addWidget(self.color[i], *(i,1))
       
        btnall = QtGui.QPushButton('Select all', self)
        btnall.clicked.connect(self.button_click_all)
        btnall.setFont(font)
        grid.addWidget(btnall, *(i+1,0))
        btnnne = QtGui.QPushButton('Select none', self)
        btnnne.clicked.connect(self.button_click_none)
        btnnne.setFont(font)
        grid.addWidget(btnnne, *(i+1,1))

        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(grid)
        
        vbox = QtGui.QVBoxLayout()
        vbox.addLayout(hbox)
        vbox.addStretch(1) 
        
        self.setLayout(vbox)

    def button_click_all(self):
        for index in range(len(self.activ)):
          self.activ[index].setCheckState(2)

    def button_click_none(self):
        for index in range(len(self.activ)):
          self.activ[index].setCheckState(0)

class Selector(QtGui.QDialog):
    def __init__(self, year, actual, colors, parent = None):
        super(Selector, self).__init__()

        self.color = actual
        
        self.output = QtGui.QComboBox()
        for item in colors:
          self.output.addItem(item)
        self.output.setFont(font)
        self.output.setStyleSheet("selection-color: black")
        
        self.output.setCurrentIndex(self.output.findText(actual, QtCore.Qt.MatchFixedString))
        
        vbox = QtGui.QVBoxLayout()
        vbox.addWidget(self.output)

        btn = QtGui.QPushButton('Ok', self)
        btn.setFont(font)
        btn.clicked.connect(self.button_click)

        hbox = QtGui.QHBoxLayout()
        hbox.addStretch(1)
        hbox.addWidget(btn)

        vbox.addLayout(hbox)
        self.setLayout(vbox)  
        self.setWindowTitle('Return: '+year)
        self.setWindowModality(QtCore.Qt.ApplicationModal)

    def button_click(self):
        self.color = str(self.output.currentText())
        self.close()        
