#!/usr/bin/python
# -*- coding: utf-8 -*-
#"""
#KOPR 1.0
#"""

import sys
import koprhand
import koprdial
import datavar
from PyQt4 import QtCore, QtGui

font = datavar.DefaultFont()

class Settings(QtGui.QWidget):
    
    def __init__(self, parent=None):
        super(Settings, self).__init__()
        
        self.initUI()
        
    def initUI(self):
        self.obscode, self.fname, self.lname, self.locations, self.telescopes, self.eyepieces, self.cameras, defaults = koprhand.InitFile()
        vbox = QtGui.QVBoxLayout()

        vbox0 = QtGui.QVBoxLayout()

        self.btn = QtGui.QPushButton('Update comet orbits', self)
        self.btn.setFont(font)
        self.btn.clicked.connect(self.button_click_comet)
        self.btn0 = QtGui.QPushButton('Update observer', self)
        self.btn0.setFont(font)
        self.btn0.clicked.connect(self.button_click_observer)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.btn)
        hbox.addWidget(self.btn0)
        vbox0.addLayout(hbox)

        lbl2 = QtGui.QLabel('Observer ICQ Code: ', self)
        lbl2.setFont(font)
        self.le2 = QtGui.QLineEdit(self)
        self.le2.setText(self.obscode)
        self.le2.setFont(font)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(lbl2)
        hbox.addWidget(self.le2)
        vbox0.addLayout(hbox)

        vbox1 = QtGui.QVBoxLayout()

        lbl1 = QtGui.QLabel('First name: ', self)
        lbl1.setFont(font)
        self.le1 = QtGui.QLineEdit(self)
        self.le1.setText(self.fname)
        self.le1.setFont(font)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(lbl1)
        hbox.addWidget(self.le1)
        vbox1.addLayout(hbox)

        lbl3 = QtGui.QLabel('Last name: ', self)
        lbl3.setFont(font)
        self.le3 = QtGui.QLineEdit(self)
        self.le3.setText(self.lname)
        self.le3.setFont(font) 
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(lbl3)
        hbox.addWidget(self.le3)
        vbox1.addLayout(hbox)

        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(vbox0)
        hbox.addLayout(vbox1)

        vbox.addLayout(hbox)

        hLine = QtGui.QFrame()
        hLine.setFrameStyle(QtGui.QFrame.HLine)
        vbox.addWidget(hLine)
        
        # location selection
        vbox0 = QtGui.QVBoxLayout()

        lblloc = QtGui.QLabel('Location: ', self)
        lblloc.setFont(font)
        self.combo2 = QtGui.QComboBox(self)
        self.combo2.setStyleSheet("selection-color: black")
        self.combo2.setFont(font)
        items, locname, lon, lat, elev, gmt = koprhand.RefreshLoc(self.locations)
        for item in items:
            self.combo2.addItem(item)
        self.combo2.setFont(font)
        self.combo2.activated.connect(self.onActivatedLoc)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(lblloc)
        hbox.addWidget(self.combo2)
        vbox0.addLayout(hbox)

        self.loclbl = QtGui.QTextEdit(self)
        self.loclbl.setFont(font)
        self.loclbl.setText('Location (GMT): '+ locname + ' (' + gmt + ')' + '\n' + 'Longitude: ' + lon + '\n'\
          + 'Latitude: ' + lat + '\n' + 'Elevation (m): ' + elev)
        self.loclbl.setReadOnly(1)
        self.loclbl.setMaximumHeight(90)
        vbox0.addWidget(self.loclbl)

        self.btnlocr = QtGui.QPushButton('Remove Location', self)
        self.btnlocr.setFont(font)
        self.btnlocr.clicked.connect(self.button_click_locrem)
        self.btnloca = QtGui.QPushButton('Add New Location', self)
        self.btnloca.setFont(font)
        self.btnloca.clicked.connect(self.button_click_addloc)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.btnlocr)
        hbox.addWidget(self.btnloca)
        vbox0.addLayout(hbox)

        # telescope selection
        vbox1 = QtGui.QVBoxLayout()

        lbl7 = QtGui.QLabel('Telescope: ', self)
        lbl7.setFont(font)
        self.combo = QtGui.QComboBox(self)
        self.combo.setStyleSheet("selection-color: black")
        self.combo.setFont(font)
        items, tname, ttype, taper, tfocl = koprhand.RefreshTel(self.telescopes)
        if ttype == 'B':
            focltext = 'Zoom: '+tfocl+'x'
        else:           
            focltext = 'Focal length: '+tfocl+' mm'
        for item in items:
            self.combo.addItem(item)
        self.combo.activated.connect(self.onActivatedTel)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(lbl7)
        hbox.addWidget(self.combo)
        vbox1.addLayout(hbox)

        self.tellbl = QtGui.QTextEdit(self)
        self.tellbl.setFont(font)
        self.tellbl.setText('Type: '+ttype + '\n' + focltext + '\n' + 'Aperture diameter: '+str(taper/10)+'-cm')
        self.tellbl.setReadOnly(1)
        self.tellbl.setMaximumHeight(90)
        vbox1.addWidget(self.tellbl)

        self.btn1 = QtGui.QPushButton('Remove Telescope', self)
        self.btn1.setFont(font)
        self.btn1.clicked.connect(self.button_click_telrem)
        self.btn6 = QtGui.QPushButton('Add New Telescope', self)
        self.btn6.setFont(font)
        self.btn6.clicked.connect(self.button_click_addtel)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.btn1)
        hbox.addWidget(self.btn6)
        vbox1.addLayout(hbox)

        # ccd selection
        lbl15 = QtGui.QLabel('CCD Camera: ', self)
        lbl15.setFont(font)
        self.combo3 = QtGui.QComboBox(self)# CCD selection
        self.combo3.setStyleSheet("selection-color: black")
        self.combo3.setFont(font)
        items, ccdcam,ccdsc,chip,chipsc,xres,yres,xpix,ypix = koprhand.RefreshCam(self.cameras) 
        for item in items:
            self.combo3.addItem(item)
        self.combo3.activated.connect(self.onActivatedCam)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(lbl15)
        hbox.addWidget(self.combo3)
        vbox0.addLayout(hbox)

        self.ccdlbl = QtGui.QTextEdit(self)
        self.ccdlbl.setFont(font)
        self.ccdlbl.setText('CCD Name: '+ccdcam+' ('+ccdsc+')' + '\n' + 'CCD Chip: '+chip+' ('+chipsc+')' + '\n'\
                  + 'Resolution: '+xres+'x'+yres+' px' + '\n' + 'Pixel size: '+xpix+'x'+ypix+' m')
        self.ccdlbl.setReadOnly(1)
        self.ccdlbl.setMaximumHeight(90)
        vbox0.addWidget(self.ccdlbl)
        
        self.btn4 = QtGui.QPushButton('Remove Camera', self)
        self.btn4.setFont(font)
        self.btn4.clicked.connect(self.button_click_ccdrem)
        self.btn5 = QtGui.QPushButton('Add New Camera', self)
        self.btn5.setFont(font)
        self.btn5.clicked.connect(self.button_click_addccd)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.btn4)
        hbox.addWidget(self.btn5)
        vbox0.addLayout(hbox)

        # Eyepiece selection
        lbl8 = QtGui.QLabel('Eyepiece: ', self)
        lbl8.setFont(font)
        self.combo1 = QtGui.QComboBox(self)
        self.combo1.setStyleSheet("selection-color: black")
        self.combo1.setFont(font)
        items, ename, efocl, efov = koprhand.RefreshEye(self.eyepieces)       
        for item in items:
            self.combo1.addItem(item)
        self.combo1.activated.connect(self.onActivatedEye)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(lbl8)
        hbox.addWidget(self.combo1)
        vbox1.addLayout(hbox)
          
        self.eyelbl = QtGui.QTextEdit(self)
        self.eyelbl.setFont(font)
        self.eyelbl.setText('Focal length: '+efocl+' mm' + '\n' + 'Apparent FOV(): '+efov)
        self.eyelbl.setReadOnly(1)
        self.eyelbl.setMaximumHeight(90)
        vbox1.addWidget(self.eyelbl)

        self.btn2 = QtGui.QPushButton('Remove Eyepiece', self)
        self.btn2.setFont(font)
        self.btn2.clicked.connect(self.button_click_eyerem)
        self.btn3 = QtGui.QPushButton('Add New Eyepiece', self)
        self.btn3.setFont(font)
        self.btn3.clicked.connect(self.button_click_addeye)
        hbox = QtGui.QHBoxLayout()
        hbox.addWidget(self.btn2)
        hbox.addWidget(self.btn3)
        vbox1.addLayout(hbox)

        # table build
        hbox = QtGui.QHBoxLayout()
        hbox.addLayout(vbox0)
        hbox.addLayout(vbox1)
        vbox.addLayout(hbox)
        
        # table end
        self.setLayout(vbox)            
        self.setWindowTitle('KOPR - Settings')    
        self.show()        

# ************************ BUTTONS *************************************************
    def button_click_comet(self):
        koprhand.CometUpdate()

    def button_click_observer(self):
        koprhand.ObsUpdate(self.le2.text()+','+self.le1.text()+','+self.le3.text())

    def button_click_locrem(self):  # remove location
        self.locations = koprhand.ItemRem(self.locations, 'loc', self.combo2.currentIndex())        
        items, locname, lon, lat, elev, gmt = koprhand.RefreshLoc(self.locations)
        self.loclbl.setText('Location (GMT): '+ locname + ' (' + gmt + ')' + '\n' + 'Longitude: ' + lon + '\n'\
          + 'Latitude: ' + lat + '\n' + 'Elevation (m): ' + elev)
        self.combo2.clear()
        for item in items:
            self.combo2.addItem(item)

    def button_click_addloc(self):   # add location
        koprdial.AddDialog('loc')  
        self.locations = koprhand.InitFile()[3] 
        items, locname, lon, lat, elev, gmt = koprhand.RefreshLoc(self.locations)
        self.loclbl.setText('Location (GMT): '+ locname + ' (' + gmt + ')' + '\n' + 'Longitude: ' + lon + '\n'\
          + 'Latitude: ' + lat + '\n' + 'Elevation (m): ' + elev)
        self.combo2.clear()
        for item in items:
            self.combo2.addItem(item)

    def button_click_telrem(self):  # remove telescope
        self.telescopes = koprhand.ItemRem(self.telescopes, 'tel', self.combo.currentIndex())        
        items, tname, ttype, taper, tfocl = koprhand.RefreshTel(self.telescopes)
        if ttype == 'B':
            focltext = 'Zoom: '+tfocl+'x'
        else:           
            focltext = 'Focal length: '+tfocl+' mm'
        self.tellbl.setText('Type: '+ttype + '\n' + focltext + '\n' + 'Aperture diameter: '+str(taper/10)+'-cm')
        self.combo.clear()
        for item in items:
            self.combo.addItem(item)
        
    def button_click_addtel(self):   # add telescoep
        koprdial.AddDialog('tel')  
        self.telescopes = koprhand.InitFile()[4] 
        items, tname, ttype, taper, tfocl = koprhand.RefreshTel(self.telescopes)
        if ttype == 'B':
            focltext = 'Zoom: '+tfocl+'x'
        else:           
            focltext = 'Focal length: '+tfocl+' mm'
        self.tellbl.setText('Type: '+ttype + '\n' + focltext + '\n' + 'Aperture diameter: '+str(taper/10)+'-cm')
        self.combo.clear()
        for item in items:
            self.combo.addItem(item)

    def button_click_eyerem(self):  # add rem eyepiece
        self.eyepieces = koprhand.ItemRem(self.eyepieces, 'eye', self.combo1.currentIndex())        
        items, ename, efocl, efov = koprhand.RefreshEye(self.eyepieces)
        self.eyelbl.setText('Focal length: '+efocl+' mm' + '\n' + 'Apparent FOV(): '+efov)
        self.combo1.clear()
        for item in items:
            self.combo1.addItem(item)

    def button_click_addeye(self):
        koprdial.AddDialog('eye')   
        self.eyepieces = koprhand.InitFile()[5]
        items, ename, efocl, efov = koprhand.RefreshEye(self.eyepieces)
        self.eyelbl.setText('Focal length: '+efocl+' mm' + '\n' + 'Apparent FOV(): '+efov)
        self.combo1.clear()
        for item in items:
            self.combo1.addItem(item)

    def button_click_ccdrem(self):   # add rem ccdcamera
        self.cameras = koprhand.ItemRem(self.cameras, 'cam', self.combo3.currentIndex())        
        items, ccdcam,ccdsc,chip,chipsc,xres,yres,xpix,ypix = koprhand.RefreshCam(self.cameras) 
        self.ccdlbl.setText('CCD Name: '+ccdcam+' ('+ccdsc+')' + '\n' + 'CCD Chip: '+chip+' ('+chipsc+')' + '\n'\
                  + 'Resolution: '+xres+'x'+yres+' px' + '\n' + 'Pixel size: '+xpix+'x'+ypix+' m')
        self.combo3.clear()
        for item in items:
            self.combo3.addItem(item)
          
    def button_click_addccd(self):
        koprdial.AddDialog('cam')  
        self.cameras = koprhand.InitFile()[6] 
        items, ccdcam,ccdsc,chip,chipsc,xres,yres,xpix,ypix = koprhand.RefreshCam(self.cameras) 
        self.ccdlbl.setText('CCD Name: '+ccdcam+' ('+ccdsc+')' + '\n' + 'CCD Chip: '+chip+' ('+chipsc+')' + '\n'\
                  + 'Resolution: '+xres+'x'+yres+' px' + '\n' + 'Pixel size: '+xpix+'x'+ypix+' m')
        self.combo3.clear()
        for item in items:
            self.combo3.addItem(item)
    
    def onActivatedLoc(self):
        items, locname, lon, lat, elev, gmt = koprhand.RefreshLoc(self.locations, self.combo2.currentIndex())
        self.loclbl.setText('Location (GMT): '+ locname + ' (' + gmt + ')' + '\n' + 'Longitude: ' + lon + '\n'\
          + 'Latitude: ' + lat + '\n' + 'Elevation (m): ' + elev)

    def onActivatedTel(self):
        items, tname, ttype, taper, tfocl = koprhand.RefreshTel(self.telescopes, self.combo.currentIndex())
        if ttype == 'B':
            focltext = 'Zoom: '+tfocl+'x'
        else:           
            focltext = 'Focal length: '+tfocl+' mm'
        self.tellbl.setText('Type: '+ttype + '\n' + focltext + '\n' + 'Aperture diameter: '+str(taper/10)+'-cm')

    def onActivatedEye(self):
        items, ename, efocl, efov = koprhand.RefreshEye(self.eyepieces, self.combo1.currentIndex())
        self.eyelbl.setText('Focal length: '+efocl+' mm' + '\n' + 'Apparent FOV(): '+efov)

    def onActivatedCam(self):
        items, ccdcam,ccdsc,chip,chipsc,xres,yres,xpix,ypix = koprhand.RefreshCam(self.cameras, self.combo3.currentIndex()) 
        self.ccdlbl.setText('CCD Name: '+ccdcam+' ('+ccdsc+')' + '\n' + 'CCD Chip: '+chip+' ('+chipsc+')' + '\n'\
                  + 'Resolution: '+xres+'x'+yres+' px' + '\n' + 'Pixel size: '+xpix+'x'+ypix+' m')

class ScrollSettings(QtGui.QScrollArea):
    """ This scroll area only handles one widget for which scrolling should be provided. """
    def __init__(self, parent=None):
        QtGui.QScrollArea.__init__(self, parent)

        self.w = Settings(self)
        self.setWidget(self.w)    
       